import wpilib
import weakref


def test_sendable_chooser():
    chooser = wpilib.SendableChooser()
    assert chooser.getSelected() is None

    chooser.setDefaultOption("option", True)
    assert chooser.getSelected() is True


def test_smart_dashboard_putdata():
    t = wpilib.Talon(4)
    ref = weakref.ref(t)
    wpilib.SmartDashboard.putData("talon", t)
    del t
    assert bool(ref) is True
    assert wpilib.SmartDashboard.getData("talon") is ref()



from . import _init_drive

# autogenerated by 'robotpy-build create-imports wpilib.drive'
from ._drive import (
    DifferentialDrive,
    KilloughDrive,
    MecanumDrive,
    RobotDriveBase,
    Vector2d,
)

__all__ = [
    "DifferentialDrive",
    "KilloughDrive",
    "MecanumDrive",
    "RobotDriveBase",
    "Vector2d",
]

del _init_drive



from ._wpilib import RobotBase
import json
import typing


def getDeployData() -> typing.Optional[typing.Dict[str, str]]:
    """
    Utility function useful for retrieving deploy-related information
    that pyfrc stores with your robot code. The dictionary has the
    following keys:

    * deploy-host
    * deploy-user
    * deploy-date
    * code-path

    If the code is deployed from a git repo, and the git program is in
    your path, the following keys will also be present:

    * git-hash
    * git-desc
    * git-branch

    :returns: None in simulation, or a dictionary
    """
    if RobotBase.isReal():
        return None

    try:
        with open("/home/lvuser/py/deploy.json") as fp:
            return json.load(fp)
    except FileNotFoundError:
        return {}



import warnings

warnings.warn(
    "wpilib.geometry has moved to wpimath.geometry", FutureWarning, stacklevel=2
)

from wpimath.geometry import Pose2d, Rotation2d, Transform2d, Translation2d, Twist2d

__all__ = ["Pose2d", "Rotation2d", "Transform2d", "Translation2d", "Twist2d"]



from wpilib import Mechanism2d


def test_create_mechanism():
    m = Mechanism2d(100, 100)
    r1 = m.getRoot("r1", 10, 10)
    l1 = r1.appendLigament("l1", 4, 3)
    l2 = l1.appendLigament("l2", 4, 3)
    assert l2 is not None

    # TODO... check that they do something?



from . import _init_interfaces

# autogenerated by 'robotpy-build create-imports wpilib.interfaces wpilib.interfaces._interfaces'
from ._interfaces import (
    Accelerometer,
    CounterBase,
    GenericHID,
    Gyro,
    MotorController,
    SpeedController,
)

__all__ = [
    "Accelerometer",
    "CounterBase",
    "GenericHID",
    "Gyro",
    "MotorController",
    "SpeedController",
]

del _init_interfaces



from . import _init_wpilib

# TODO: robotpy-build subpackage bug
from wpimath._controls._controls import trajectory as _

# autogenerated by 'robotpy-build create-imports wpilib'
from ._wpilib import (
    ADIS16448_IMU,
    ADIS16470_IMU,
    ADXL345_I2C,
    ADXL345_SPI,
    ADXL362,
    ADXRS450_Gyro,
    AddressableLED,
    AnalogAccelerometer,
    AnalogEncoder,
    AnalogGyro,
    AnalogInput,
    AnalogOutput,
    AnalogPotentiometer,
    AnalogTrigger,
    AnalogTriggerOutput,
    AnalogTriggerType,
    BuiltInAccelerometer,
    CAN,
    CANData,
    CANStatus,
    Color,
    Color8Bit,
    Compressor,
    CompressorConfigType,
    Counter,
    DataLogManager,
    DMC60,
    DSControlWord,
    DigitalGlitchFilter,
    DigitalInput,
    DigitalOutput,
    DigitalSource,
    DoubleSolenoid,
    DriverStation,
    DutyCycle,
    DutyCycleEncoder,
    Encoder,
    Field2d,
    FieldObject2d,
    I2C,
    IterativeRobotBase,
    Jaguar,
    Joystick,
    LiveWindow,
    Mechanism2d,
    MechanismLigament2d,
    MechanismObject2d,
    MechanismRoot2d,
    MotorControllerGroup,
    MotorSafety,
    NidecBrushless,
    Notifier,
    PS4Controller,
    PWM,
    PWMMotorController,
    PWMSparkMax,
    PWMTalonFX,
    PWMTalonSRX,
    PWMVenom,
    PWMVictorSPX,
    PneumaticHub,
    PneumaticsBase,
    PneumaticsControlModule,
    PneumaticsModuleType,
    PowerDistribution,
    Preferences,
    Relay,
    RobotBase,
    RobotController,
    RobotState,
    RuntimeType,
    SD540,
    SPI,
    SendableBuilderImpl,
    SendableChooser,
    SendableChooserBase,
    SensorUtil,
    SerialPort,
    Servo,
    SmartDashboard,
    Solenoid,
    Spark,
    SynchronousInterrupt,
    Talon,
    TimedRobot,
    Timer,
    TimesliceRobot,
    Tracer,
    Ultrasonic,
    Victor,
    VictorSP,
    Watchdog,
    XboxController,
    getCurrentThreadPriority,
    getErrorMessage,
    getTime,
    setCurrentThreadPriority,
    wait,
)

__all__ = [
    "ADIS16448_IMU",
    "ADIS16470_IMU",
    "ADXL345_I2C",
    "ADXL345_SPI",
    "ADXL362",
    "ADXRS450_Gyro",
    "AddressableLED",
    "AnalogAccelerometer",
    "AnalogEncoder",
    "AnalogGyro",
    "AnalogInput",
    "AnalogOutput",
    "AnalogPotentiometer",
    "AnalogTrigger",
    "AnalogTriggerOutput",
    "AnalogTriggerType",
    "BuiltInAccelerometer",
    "CAN",
    "CANData",
    "CANStatus",
    "Color",
    "Color8Bit",
    "Compressor",
    "CompressorConfigType",
    "Counter",
    "DataLogManager",
    "DMC60",
    "DSControlWord",
    "DigitalGlitchFilter",
    "DigitalInput",
    "DigitalOutput",
    "DigitalSource",
    "DoubleSolenoid",
    "DriverStation",
    "DutyCycle",
    "DutyCycleEncoder",
    "Encoder",
    "Field2d",
    "FieldObject2d",
    "I2C",
    "IterativeRobotBase",
    "Jaguar",
    "Joystick",
    "LiveWindow",
    "Mechanism2d",
    "MechanismLigament2d",
    "MechanismObject2d",
    "MechanismRoot2d",
    "MotorControllerGroup",
    "MotorSafety",
    "NidecBrushless",
    "Notifier",
    "PS4Controller",
    "PWM",
    "PWMMotorController",
    "PWMSparkMax",
    "PWMTalonFX",
    "PWMTalonSRX",
    "PWMVenom",
    "PWMVictorSPX",
    "PneumaticHub",
    "PneumaticsBase",
    "PneumaticsControlModule",
    "PneumaticsModuleType",
    "PowerDistribution",
    "Preferences",
    "Relay",
    "RobotBase",
    "RobotController",
    "RobotState",
    "RuntimeType",
    "SD540",
    "SPI",
    "SendableBuilderImpl",
    "SendableChooser",
    "SendableChooserBase",
    "SensorUtil",
    "SerialPort",
    "Servo",
    "SmartDashboard",
    "Solenoid",
    "Spark",
    "SynchronousInterrupt",
    "Talon",
    "TimedRobot",
    "Timer",
    "TimesliceRobot",
    "Tracer",
    "Ultrasonic",
    "Victor",
    "VictorSP",
    "Watchdog",
    "XboxController",
    "getCurrentThreadPriority",
    "getErrorMessage",
    "getTime",
    "setCurrentThreadPriority",
    "wait",
]

# Error reporting
from ._impl.report_error import reportError, reportWarning

__all__ += ["reportError", "reportWarning"]

# deprecated alias
SpeedControllerGroup = MotorControllerGroup

del _init_wpilib

from .cameraserver import CameraServer
from .deployinfo import getDeployData

try:
    from .version import version as __version__
except ImportError:
    __version__ = "master"

from ._impl.main import run

__all__ += ["CameraServer", "run"]
